library(igraph)
library(pforeach)
source("functions.R")

n.core.candidates <- 5
n.functions <- 2
start.phenotype <- 9
n.phenotype <- 3
alpha <- c(1, -1)
beta <- c(2, 1)
color.order <- c("red", "dodgerblue3", "forestgreen", "purple", "yellow", "orange", "cyan", "chocolate4", "antiquewhite", "lightpink")

otu.info <- read.table("taxa_appl3.txt", header=T)
mat <- as.matrix(read.table("network_appl3.txt", header=T, row.name=1))

otu.info.selected <- c()
for (i in 1:ncol(mat)){
  otu.info.selected <- rbind(otu.info.selected, subset(otu.info, otu.info[,1]==colnames(mat)[i]))
}

mb.graph.positive <- mat > 0
mb.graph.negative <- mat < 0

ig.mb.positive <- graph.adjacency(mb.graph.positive, mode='undirected', diag=FALSE)
ig.mb.negative <- graph.adjacency(mb.graph.negative, mode='undirected', diag=FALSE)

g <- ig.mb.positive
v <- components(g)
v$csize


######The largest connected graph######
g1 <- induced.subgraph(graph = g, vids = which(v$membership == order(v$csize, decreasing=T)[1]))
otu.info.selected.g1 <- otu.info.selected[which(v$membership == order(v$csize, decreasing=T)[1]), ]

memb.g1 <- community.imp(g1)
memb.info.g1 <- cbind(otu.info.selected.g1, memb.g1)

# positive
f <- otu.info.selected.g1[, (ncol(otu.info.selected) - (n.functions-1)):ncol(otu.info.selected) ]

### set gamma and delta
b <- func.btw.final(g1, f, alpha, beta, gamma=1, delta=0)
otu.info.selected.posi.g1 <- transform(otu.info.selected.g1, module = memb.g1, betweenness.func = b[[1]], b[[2]])
#otu.info.selected.posi.g1[order(otu.info.selected.posi.g1$betweenness.func, decreasing = T), ]

write.table(otu.info.selected.posi.g1, file="OTUs.results.g1.txt", row.names=F, quote=F, sep="\t")


result <- list()
#positive
for(i in 1:max(memb.g1)){
  subd <- subset(otu.info.selected.posi.g1, module == i)
  result <- c(result, list(subd[order(subd$betweenness.func, decreasing = T)[1:n.core.candidates], ]))
}

for (i in 1:max(memb.g1)) {
  write.table(result[[i]], file=sprintf("Core.g1.Module_%s.txt", i), quote=F, row.name=F, sep="\t")
}

## Drawing a networklibrary(RColorBrewer)
color.order <- color.order[1:max(memb.g1)]

vcol <- c()
for (i in 1:length(memb.g1)) {
	vcol[i] <- color.order[memb.g1[i]]
}

#vshape <- as.character(otu.info.selected.posi.g1$Shape)
#vcol.taxon <- as.character(otu.info.selected.posi.g1$Color)

size.g1 <- unlist(b[1])
size.g1[which(size.g1 < 0)] <- 0
sort(size.g1)

am.coord <- layout.fruchterman.reingold(g1)

#par(mfrow=c(1,2))

plot(g1, layout=am.coord, vertex.size=4*((1 + size.g1)^0.2), vertex.label="", main="MB.positive", edge.color="skyblue", vertex.color=vcol)

legend("topright", legend=1:max(memb.g1), fill=color.order)

## host phenotype score for each module

host.phenotype <- matrix(NA, nrow=max(memb.g1), ncol=(n.phenotype+1))
colnames(host.phenotype) <- c("Module", colnames(memb.info.g1)[start.phenotype:(start.phenotype + n.phenotype -1)])
host.phenotype[,1] <- 1:max(memb.g1)

for (j in 1:n.phenotype) {
for (i in 1:max(memb.g1)) {
	host.phenotype[i,j+1] <- mean(subset(memb.info.g1[,(start.phenotype + j -1)], memb.info.g1$memb.g1==i))
}}

write.table(host.phenotype, file="Host.Phenotype.g1.Module.txt", row.names=F, quote=F, sep="\t")



######The second largest connected graph######
g2 <- induced.subgraph(graph = g, vids = which(v$membership == order(v$csize, decreasing=T)[2]))
otu.info.selected.g2 <- otu.info.selected[which(v$membership == order(v$csize, decreasing=T)[2]), ]

memb.g2 <- community.imp(g2)
memb.info.g2 <- cbind(otu.info.selected.g2, memb.g2)

# positive
f <- otu.info.selected.g2[, (ncol(otu.info.selected) - (n.functions-1)):ncol(otu.info.selected) ]

### set gamma and delta
b <- func.btw.final(g2, f, alpha, beta, gamma=1, delta=0)
otu.info.selected.posi.g2 <- transform(otu.info.selected.g2, module = memb.g2, betweenness.func = b[[1]], b[[2]])
#otu.info.selected.posi.g2[order(otu.info.selected.posi.g2$betweenness.func, decreasing = T), ]

write.table(otu.info.selected.posi.g2, file="OTUs.results.g2.txt", row.names=F, quote=F, sep="\t")


result <- list()
#positive
for(i in 1:max(memb.g2)){
  subd <- subset(otu.info.selected.posi.g2, module == i)
  result <- c(result, list(subd[order(subd$betweenness.func, decreasing = T)[1:n.core.candidates], ]))
}

for (i in 1:max(memb.g2)) {
  write.table(result[[i]], file=sprintf("Core.g2.Module_%s.txt", i), quote=F, row.name=F, sep="\t")
}

## Drawing a networklibrary(RColorBrewer)
color.order <- color.order[1:max(memb.g2)]

vcol <- c()
for (i in 1:length(memb.g2)) {
	vcol[i] <- color.order[memb.g2[i]]
}

#vshape <- as.character(otu.info.selected.posi.g2$Shape)
#vcol.taxon <- as.character(otu.info.selected.posi.g2$Color)

size.g2 <- unlist(b[1])
size.g2[which(size.g2 < 0)] <- 0
sort(size.g2)

am.coord <- layout.fruchterman.reingold(g2)

#par(mfrow=c(1,2))

plot(g2, layout=am.coord, vertex.size=12*((0.5 + size.g2)^0.2), vertex.label="", main="MB.positive", edge.color="skyblue", vertex.color=vcol)

legend("topright", legend=1:max(memb.g2), fill=color.order)

## host phenotype score for each module

host.phenotype <- matrix(NA, nrow=max(memb.g2), ncol=(n.phenotype+1))
colnames(host.phenotype) <- c("Module", colnames(memb.info.g2)[start.phenotype:(start.phenotype + n.phenotype -1)])
host.phenotype[,1] <- 1:max(memb.g2)

for (j in 1:n.phenotype) {
for (i in 1:max(memb.g2)) {
	host.phenotype[i,j+1] <- mean(subset(memb.info.g2[,(start.phenotype + j -1)], memb.info.g2$memb.g2==i))
}}

write.table(host.phenotype, file="Host.Phenotype.g2.Module.txt", row.names=F, quote=F, sep="\t")

